@@ -9,6 +9,7 @@
 #include <asm/clocksource.h>
 #endif
 #include <lttng/kernel-version.h>
+#include <wrapper/kvm-x86.h>
 #include <../arch/x86/kvm/lapic.h>
 #include <../arch/x86/kvm/kvm_cache_regs.h>
 
@@ -115,7 +116,40 @@ LTTNG_TRACEPOINT_EVENT_MAP(kvm_apic, kvm_x86_apic,
 /*
  * Tracepoint for kvm guest exit:
  */
-#if (LTTNG_LINUX_VERSION_CODE >= LTTNG_KERNEL_VERSION(5,16,0))
+#if (LTTNG_LINUX_VERSION_CODE >= LTTNG_KERNEL_VERSION(5,18,0))
+LTTNG_TRACEPOINT_EVENT_CODE_MAP(kvm_exit, kvm_x86_exit,
+	TP_PROTO(struct kvm_vcpu *vcpu, u32 isa),
+	TP_ARGS(vcpu, isa),
+
+	TP_locvar(
+		u32 exit_reason;
+		u64 info1, info2;
+		u32 intr_info, error_code;
+	),
+
+	TP_code_pre(
+		lttng_kvm_x86_get_exit_info(vcpu,
+				&tp_locvar->exit_reason,
+				&tp_locvar->info1,
+				&tp_locvar->info2,
+				&tp_locvar->intr_info,
+				&tp_locvar->error_code);
+	),
+
+	TP_FIELDS(
+		ctf_integer(unsigned int, exit_reason, tp_locvar->exit_reason)
+		ctf_integer(unsigned long, guest_rip, kvm_rip_read(vcpu))
+		ctf_integer(u32, isa, isa)
+		ctf_integer(u64, info1, tp_locvar->info1)
+		ctf_integer(u64, info2, tp_locvar->info2)
+		ctf_integer(u32, intr_info, tp_locvar->intr_info)
+		ctf_integer(u32, error_code, tp_locvar->error_code)
+		ctf_integer(unsigned int, vcpu_id, vcpu->vcpu_id)
+	),
+
+	TP_code_post()
+)
+#elif (LTTNG_LINUX_VERSION_CODE >= LTTNG_KERNEL_VERSION(5,16,0))
 LTTNG_TRACEPOINT_EVENT_CODE_MAP(kvm_exit, kvm_x86_exit,
 	TP_PROTO(struct kvm_vcpu *vcpu, u32 isa),
 	TP_ARGS(vcpu, isa),
@@ -535,7 +569,7 @@ LTTNG_TRACEPOINT_EVENT_MAP(kvm_emulate_insn, kvm_x86_emulate_insn,
 				vcpu->arch.emulate_ctxt.fetch.data)
 		ctf_array(__u8, insn, vcpu->arch.emulate_ctxt.fetch.data, 15)
 		ctf_integer(__u8, flags, kei_decode_mode(vcpu->arch.emulate_ctxt.mode))
-#else
+#elif (LTTNG_LINUX_VERSION_CODE < LTTNG_KERNEL_VERSION(5,18,0))
 		ctf_integer(__u64, rip, vcpu->arch.emulate_ctxt->_eip -
 				(vcpu->arch.emulate_ctxt->fetch.ptr -
 					vcpu->arch.emulate_ctxt->fetch.data))
@@ -544,6 +578,15 @@ LTTNG_TRACEPOINT_EVENT_MAP(kvm_emulate_insn, kvm_x86_emulate_insn,
 				vcpu->arch.emulate_ctxt->fetch.data)
 		ctf_array(__u8, insn, vcpu->arch.emulate_ctxt->fetch.data, 15)
 		ctf_integer(__u8, flags, kei_decode_mode(vcpu->arch.emulate_ctxt->mode))
+#else
+		ctf_integer(__u64, rip, vcpu->arch.emulate_ctxt->_eip -
+				(vcpu->arch.emulate_ctxt->fetch.ptr -
+					vcpu->arch.emulate_ctxt->fetch.data))
+		ctf_integer(__u32, csbase, lttng_kvm_x86_get_segment_base(vcpu, VCPU_SREG_CS))
+		ctf_integer(__u8, len, vcpu->arch.emulate_ctxt->fetch.ptr -
+				vcpu->arch.emulate_ctxt->fetch.data)
+		ctf_array(__u8, insn, vcpu->arch.emulate_ctxt->fetch.data, 15)
+		ctf_integer(__u8, flags, kei_decode_mode(vcpu->arch.emulate_ctxt->mode))
 #endif
 		ctf_integer(__u8, failed, failed)
 	)
